package openfuture.legend;

import java.awt.*;

/**
 * Class JLegend is used to display a Legend for some Business Graphics
 *
 * @author  Markus Giebeler
 * @version $Revision: 1.0 $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html"
 */
public class JLegend extends Panel {
  // constants
  public static int ROWS = 0;
  public static int COLS = 1;

  protected static int SIZE = 14;
  protected static int ROW_INCREMENT = 1;
  protected static int COL_INCREMENT = 20;
  protected static int OFFSET = 15;
  protected static int SPACE = 2;

  // static fields
  /** Field defaultColors (Color[]) defines some default colors
      if none or not enought colors are provided */
  private static Color[] defaultColors;

  // attributes
  /** Field labels (String[]) holds the label texts */
  private String[] labels = new String[0];
  /** Field colors (Color[]) the label Colors */
  private Color[] colors = new Color[0];
  /** Field orientation (int) indicates the orientatin of the legend */
  private int orientation = ROWS;
  /** Field borderVisible (boolean) determines if border of boxes is visible */
  private boolean borderVisible = true;

 /** Construction */
  public JLegend() {
  }

 /**
  * description
  *
  * @param  g (Graphics) description
  */
  public void paint(Graphics g) {
    // start values
    int maxWidth  = getMaxWidth();
    int maxHeight = getMaxHeight();
    int amount = labels.length;
    int row = 0;
    int column = 0;

    // now draw graphic
    if(orientation==COLS) {
      // draw columns
      for(int j=0; j<amount; j++) {
        if(j>=getColors().length) {
          g.setColor(getDefaultColor(j));
        } else {
          g.setColor(getColors()[j]);
        }
        g.fillRect(column + SPACE, row + SPACE, SIZE, SIZE);
        g.setColor(Color.black);
        if(isBorderVisible()) {
          g.drawRect(column + SPACE, row + SPACE, SIZE, SIZE);
        }
        g.drawString(labels[j], column + COL_INCREMENT, row + OFFSET);
        row += maxHeight + ROW_INCREMENT;
        // next column
        if(row>getSize().getHeight() - ROW_INCREMENT - maxHeight) {
          row=0;
          column += maxWidth + COL_INCREMENT;
        }
      }
    } else {
      // draw rows
      for(int j=0; j<amount; j++) {
        if(j>=getColors().length) {
          g.setColor(getDefaultColor(j));
        } else {
          g.setColor(getColors()[j]);
        }
        g.fillRect(column + SPACE, row + SPACE, SIZE, SIZE);
        g.setColor(Color.black);
        if(isBorderVisible()) {
          g.drawRect(column + SPACE, row + SPACE, SIZE, SIZE);
        }
        g.drawString(labels[j], column + COL_INCREMENT, row + OFFSET);
        column += maxWidth + COL_INCREMENT;
        // next row
        if(column>getSize().getWidth() - COL_INCREMENT - maxWidth) {
          column=0;
          row += maxHeight + ROW_INCREMENT;
        }
      }
    }
  }

 /**
  * get the maximal label width
  *
  * @return (int) max label width
  */
  protected int getMaxWidth() {
    FontMetrics fm = getFontMetrics(getFont());

    // get max label width
    int amount = labels.length;
    int maxWidth = 10;
    for(int i=0; i<amount; i++) {
      int sw = fm.stringWidth(labels[i]);
      if(sw > maxWidth) {
        maxWidth = sw;
      }
    }
    return maxWidth;
  }

 /**
  * get the maximal label height
  *
  * @return (int) max label height
  */
  protected int getMaxHeight() {
    FontMetrics fm = getFontMetrics(getFont());
    return fm.getHeight();
  }

 /**
  * get some Color for an int value
  *
  * @param  index (int) index of requested Color
  * @return (Color) a Color
  */
  protected Color getDefaultColor(int index) {
    int length = getDefaultColors().length;
    int i = index % length;
    return getDefaultColors()[i];
  }

 /**
  * Return contents of field defaultColors
  *
  * @return (Color[]) contents of field defaultColors
  */
  public final static Color[] getDefaultColors() {
    if(defaultColors==null) {
      Color[] c = {
        new Color(225,0,0),     // red
        new Color(0,51,190),    // blue
        new Color(0,190,51),    // green
        new Color(255,56,140),  // pink
        new Color(255,255,0),   // yellow

        new Color(140,0,0),     // dark red
        new Color(0,51,127),    // dark blue
        new Color(0,127,51),    // dark green
        new Color(135,32,120),  // dark purple
        new Color(255,160,0),   // dark orange

        new Color(255,120,120), // light red
        new Color(99,160,255),  // light blue
        new Color(99,255,99),   // light green
        new Color(255,160,220), // light pink
        new Color(255,255,140), // light yellow

      };
      defaultColors = c;
    }
    return defaultColors;
  }

 /**
  * Set contents of field defaultColors to the given value
  *
  * @param newValue (Color[]) new value for field defaultColors
  */
  public final static void setDefaultColors(Color[] newValue) {
    defaultColors = newValue;
  }

 /**
  * Return contents of field labels
  *
  * @return (String[]) contents of field labels
  */
  public final String[] getLabels() {
    return labels;
  }

 /**
  * Set contents of field labels to the given value
  *
  * @param newValue (String[]) new value for field labels
  */
  public final void setLabels(String[] newValue) {
    this.labels = newValue;
    repaint();
  }

 /**
  * Return contents of field colors
  *
  * @return (Color[]) contents of field colors
  */
  public final Color[] getColors() {
    return colors;
  }

 /**
  * Set contents of field colors to the given value
  *
  * @param newValue (Color[]) new value for field colors
  */
  public final void setColors(Color[] newValue) {
    this.colors = newValue;
    repaint();
  }

 /**
  * Return contents of field orientation
  *
  * @return (int) contents of field orientation
  */
  public final int getOrientation() {
    return orientation;
  }

 /**
  * Set contents of field orientation to the given value
  *
  * @param newValue (int) new value for field orientation
  */
  public final void setOrientation(int newValue) {
    this.orientation = newValue;
    repaint();
  }

 /**
  * Return contents of field borderVisible
  * 
  * @return (boolean) contents of field borderVisible
  */
  public final boolean isBorderVisible() {
    return borderVisible;
  }

 /**
  * Set contents of field borderVisible to the given value
  * 
  * @param newValue (boolean) new value for field borderVisible
  */
  public final void setBorderVisible(boolean newValue) {
    this.borderVisible = newValue;
    repaint();
  }
}
